/**
 * Program FileTransferBean.java.
 * Comments:  This class is used to move files back and forth between the 
 * localhost and the remote server (the machine running the WebFlow server).
 * Uses the RemoteFile module to do this.  This is a rewrite of the original 
 * version by Tom Haupt.
 *
 * Note that this is a _client_ program.
 */

package WebFlow.GridEvent;

import org.omg.CORBA.*;
import java.io.*;
import WebFlow.RemoteFile.*;
import WebFlow.RemoteFile.RemoteFilePackage.*;


public class FileTransfer {
    private ORB orb;
    private RemoteFile file;
    private short block_size = 8912;
    
    /**
     * FileTransfer constructor accepts ORB and RemoteFile IOR
     **/
    public FileTransfer(ORB anOrb, String obj_ior)
    {
	orb = anOrb;
	org.omg.CORBA.Object obj = orb.string_to_object(obj_ior);
	file = RemoteFileHelper.narrow(obj);
    }
    
    /**
     * Use this contstructor to pass a preexisting RemoteFile object
     * from another class.
     */
    public FileTransfer(RemoteFile rf)
    {
	file = rf;
    }  
    
    /**
     * FileSend sends local file to the remote host
     * @param source name of the local file to be sent
     * @param dest name of the remote file
     **/
    public void FileSend(String localFile, String remoteFile) 
	throws FileNotFoundException, FileException, IOException {

	//open local file 
	boolean test=true;
	File sfile = new File(localFile);
	FileInputStream source_file = new FileInputStream(sfile);
	long file_size = sfile.length();
	long pos = 0;
	byte buffer[] = new byte[block_size]; //create temporary buffer 
	try { 
	    //open remote file to be written 
	    file.open(remoteFile, "rw"); 
	    while(test) { 
		//read data from the local file 
		int icount = source_file.read(buffer); 
		if(icount == -1) 
		    throw new EOFException(); 
		byte arr[]; 
		if(block_size == icount) {
		    arr = buffer; 
		    test=false;
		    file.close();
		    source_file.close();
		}
		else { 
		    //create buffer to be sent to the remote host 
		    arr = new byte[icount]; 
		    //copy temporary buffer 
		    for(int i = 0; i < icount; i++) 
			arr[i] = buffer[i]; 
		} 
		file.putBlock(arr); //write data to the remote file 
		pos += icount; 
	    }
	}
	catch(EOFException ex) { 
	    file.close();             
	    source_file.close(); 
	} 
	catch(IOException ex) { 
	    file.close(); 
	    throw ex; 
	} 
    } 
    
    /**
     * FileReceive copies file from the remote host to the local file
     * @param dest name of the local file to copy to
     * @param source name of the remote file to be copied
     **/
    public void FileReceive(String localFile, String remoteFile) 
	throws FileException, IOException {

	//open local file for writing
	java.io.FileOutputStream dest_file=new FileOutputStream(localFile);
	//create buffer to hold read data
	byte arr[] = null;
	try {
	    file.open(remoteFile, "r"); //open remote file for reading
	    long file_size = file.getFileSize();
	    long pos = 0;
	    while(true) {
		arr = file.getBlock();
		if(arr.length > 0) {
		    dest_file.write(arr); //write block to the local file
		    pos += arr.length;
		}
		else throw new EOFFileException();
	    }
	}
	catch(EOFFileException ex) {
	    dest_file.flush(); //flush buffers
	    dest_file.close();
	    file.close();
	}
    }
    
    /** 
     * SetBlockSize sets size of the block of data to be sent for each 
     * transfer 
     * @param aSize new size of the block 
     **/ 
    public void SetBlockSize(short aSize) 
    { 
	block_size = aSize; 
	file.setBlockSize(aSize); 
    } 
} 







